"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const _ = require('lodash');
const axios = require('axios').default;
const querystring = require('querystring');
class SwarmappApi {
    constructor(oauth_token) {
        this.config = {
            m: 'swarm',
            v: '20221101',
            // A random coordinate to use between calls imitating regular behavior
            ll: this.createLatLngString('26.30' + this.between(340000000000, 499999999999), '50.1' + this.between(2870000000000, 3119999999999)),
            altAcc: "30.000000",
            llAcc: "14.825392",
            floorLevel: "2146959360",
            alt: "12.275661"
        };
        if (oauth_token) {
            this.config.oauth_token = oauth_token;
        }
        this.basePath = 'https://api.foursquare.com/v2/';
        this.headers = {
            'User-Agent': 'com.foursquare.robin.ios.phone:20230316.2230.52:20221101:iOS 16.1.1:iPhone13,4'
        };
        this.initialize();
    }
    initialize() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.getUser();
                this.user = (_b = (_a = response === null || response === void 0 ? void 0 : response.data) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.user;
                this.log("hello");
            }
            catch (error) {
                this.error("Could not authenticate user");
            }
        });
    }
    getLL() {
        return this.config.ll;
    }
    log(message) {
        var _a, _b;
        console.log(`${new Date().toLocaleString()} - ${(_a = this === null || this === void 0 ? void 0 : this.user) === null || _a === void 0 ? void 0 : _a.firstName}(${(_b = this === null || this === void 0 ? void 0 : this.user) === null || _b === void 0 ? void 0 : _b.id}) - `, message);
    }
    error(message) {
        var _a, _b;
        console.error(`${new Date().toLocaleString()} - ${(_a = this === null || this === void 0 ? void 0 : this.user) === null || _a === void 0 ? void 0 : _a.firstName}(${(_b = this === null || this === void 0 ? void 0 : this.user) === null || _b === void 0 ? void 0 : _b.id}) - Error:`, message);
    }
    initiatemultifactorlogin(username, password, client_id, client_secret) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                client_id: client_id,
                client_secret: client_secret,
                username: username,
                password: password,
            };
            try {
                const response = yield axios.post(this.basePath + '/private/initiatemultifactorlogin', null, { params });
                this.flowId = response.data.flowId;
                return response.data.access_token;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    completemultifactorlogin(code, client_id, client_secret) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                client_id: client_id,
                client_secret: client_secret,
                code: code,
                flowId: this.flowId
            };
            try {
                const response = yield axios.post(this.basePath + '/private/completemultifactorlogin', null, { params });
                this.config.oauth_token = response.data.oauth_token;
                return response.data.access_token;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Get Commands
    // TODO: requires testing, does not accept non swarmapp (mobile) oauth tokens
    getFriends(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/friends', { 'params': this.config });
                return result.data.response.friends;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: works only for foursquare client, requires more testing
    getFollowings(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.m = 'foursquare';
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/following', { 'params': this.config });
                return result.data.response.following;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: works only for foursquare client, requires more testing
    getFollowers(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.m = 'foursquare';
            this.config.user_id = user_id;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/followers', { 'params': this.config });
                return result.data.response.followers;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // TODO: should get user, then get checkins and location of last check-in
    getLastSeen(user_id = 'self', limit = 100) {
        this.config.user_id = user_id;
        this.config.limit = limit;
        try {
            return this.getUser(user_id);
        }
        catch (error) {
            this.error(error);
        }
    }
    getUser(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield axios.get(`${this.basePath}users/${user_id}`, { 'params': this.config });
                return result;
            }
            catch (error) {
                throw new Error("Error getting user data, maybe an authentication error ?");
                return;
            }
        });
    }
    getVenue(venue_id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield axios.get(`${this.basePath}venues/${venue_id}/`, { 'params': this.config });
                return result.data.response.venue;
            }
            catch (error) {
                throw new Error("Error getting venue data, maybe an authentication error ?");
                return;
            }
        });
    }
    getCheckins(user_id = 'self', limit = 100, afterTimestamp) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof afterTimestamp !== 'undefined') {
                this.config.afterTimeStamp = afterTimestamp;
            }
            this.config.user_id = user_id;
            this.config.limit = limit;
            try {
                const result = yield axios.get(this.basePath + 'users/' + user_id + '/checkins', { 'params': this.config });
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // not sure if it is working
    getGeos(user_id = 'self') {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.user_id = user_id;
            delete this.config.afterTimeStamp;
            try {
                const response = yield axios.get(this.basePath + '/users/' + user_id + '/map', { 'params': this.config });
                return response.data.response;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // returns user timeline after timestamp
    getRecent(limit = 100, ll) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.limit = limit;
            this.config.afterTimeStamp = (Math.floor(Date.now() / 1000) - (1 * 24 * 60 * 60)).toString();
            if (typeof ll !== 'undefined') {
                this.log('found location');
                this.config.ll = ll;
            }
            try {
                const result = yield axios.get(this.basePath + 'checkins/recent', {
                    params: this.config,
                    paramsSerializer: (params) => {
                        return querystring.stringify(params);
                    }
                });
                return result.data.response.recent;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Get Trending Venues
    getTrending(limit = 50, ll, near, radius = 100000) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.limit = limit;
            this.config.radius = radius;
            if (typeof ll !== 'undefined') {
                this.config.ll = ll;
                // just make any call to mimic updating location
                //await this.getUser(); 
            }
            else {
                this.config.near = near;
            }
            // not sure how to mimic user location here
            // it might not be needed
            try {
                const result = yield axios.get(this.basePath + 'venues/trending', {
                    params: this.config,
                    paramsSerializer: (params) => {
                        return querystring.stringify(params);
                    }
                });
                return result.data.response.venues;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Checkin Functions
    checkIn(venue_id, silent) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (silent) {
                this.config.broadcast = 'private';
            }
            this.config.venueId = venue_id;
            // probably updates user LL
            const venue_info = yield this.getVenue(venue_id);
            this.config.ll = this.createLatLngString(venue_info.location.lat, venue_info.location.lng);
            try {
                const result = yield axios.post(this.basePath + 'checkins/add', querystring.stringify(this.config));
                const checkin = (_a = result.data.response) === null || _a === void 0 ? void 0 : _a.checkin;
                return checkin;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // Suspect this method to update user location, doesn't work
    private_log() {
        return __awaiter(this, void 0, void 0, function* () {
            const FormData = require('form-data');
            const formData = new FormData();
            formData.append('altAcc', this.config.altAcc);
            formData.append('llAcc', this.config.llAcc);
            formData.append('floorLevel', this.config.floorLevel);
            formData.append('alt', this.config.alt);
            formData.append('csid', '1243');
            formData.append('v', this.config.v);
            formData.append('oauth_token', this.config.oauth_token);
            formData.append('m', this.config.m);
            formData.append('ll', this.config.ll);
            formData.append('background', 'true');
            const script = [{ "name": "ios-metrics", "csid": 1243, "ctc": 4, "metrics": [{ "rid": "642a1e4371f95f2eb16b8eda", "n": "SwarmTimelineFeedViewController", "im": "3-3", "vc": 1 }], "event": "background", "cts": 1680481860.378182 }];
            const stringified = JSON.stringify(script);
            formData.append('loglines', stringified, { filename: 'loglines', contentType: 'application/json' });
            //console.log(formData);
            try {
                const result = yield axios.post(this.basePath + 'private/log', formData, {
                    headers: Object.assign({}, formData.getHeaders())
                });
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    register_device() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                limitAdsTracking: '1',
                carrier: 'stc',
                hasWatch: '1',
                csid: '1242',
                alt: '11.791945',
                llAcc: '14.825392',
                otherDeviceIds: '6054750ee01fbc1e3492a745,61187d1fceb2110097a0fc02',
                ll: '21.530136,39.172863',
                altAcc: '17.547791',
                locationAuthorizationStatus: 'authorizedwheninuse',
                token: 'ec0718c5d042b63587c14d461bb077d1262811456c4799558e1df2616f02cc9a',
                oauth_token: 'DW233H4JWJZ0E14QU01LFWUZL4RDQPLAF10BAJEI2FTWNOKH',
                iosSettings: '0',
                m: 'swarm',
                floorLevel: '2146959360',
                measurementSystem: 'metric',
                uniqueDevice: '6054750ee01fbc1e3492a745',
                v: '20221101',
                backgroundRefreshStatus: 'available',
            };
            try {
                const result = yield axios.post(this.basePath + '/private/registerdevice ', querystring.stringify(params));
                const registeration = (_a = result.data.response) === null || _a === void 0 ? void 0 : _a.checkin;
                return registeration;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    addHereNow(checkin, females_only = true) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const hereNow = (_a = checkin === null || checkin === void 0 ? void 0 : checkin.venue) === null || _a === void 0 ? void 0 : _a.hereNow;
            this.log(`Adding hereNows from ${checkin.venue.name}`);
            this.log(`Females only setting is ${females_only}`);
            this.log(`Found ${hereNow.count} people checked in classified under ${hereNow.groups.length} groups`);
            if (hereNow.count > 0) {
                for (const group of hereNow.groups) {
                    this.log(`Found ${group.count} in group ${group.name}`);
                    if (group.count > 0) {
                        for (const item of group.items) {
                            if (females_only) {
                                if (((_b = item.user) === null || _b === void 0 ? void 0 : _b.gender) == 'female') {
                                    yield this.addFriendByID(item.user.id);
                                }
                            }
                            else {
                                yield this.addFriendByID(item.user.id);
                            }
                        }
                    }
                }
            }
            else {
                return 'no one is here';
            }
        });
    }
    addFriendByID(user_id) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.getUser(user_id.toString());
                const new_friend = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.user;
                this.log(`Checking ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.firstName}`);
                //console.log(new_friend);
                this.log(`Relationship status is ${new_friend.relationship}`);
                if (new_friend.relationship == 'none') {
                    const result = yield axios.post(this.basePath + 'users/' + user_id + '/request', querystring.stringify(this.config));
                    this.log(`Added ${new_friend === null || new_friend === void 0 ? void 0 : new_friend.firstName}`);
                    return result.data.response.user.relationship;
                }
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    // auto add trending function
    autoAddTrending(location_name, limit_trending) {
        return __awaiter(this, void 0, void 0, function* () {
            const trending = yield this.getTrending(limit_trending, undefined, location_name);
            for (const venue of trending) {
                if (venue.hereNow.count > 0) {
                    this.log(`venue location is ${venue.location.lat} ${venue.location.lng}`);
                    let checkin = yield this.checkIn(venue.id, true);
                    this.log(`my ll after checkin is ${this.getLL()}`);
                    yield this.addHereNow(checkin);
                }
            }
        });
    }
    // Like Functions
    likeCheckin(checkin_id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield axios.post(this.basePath + 'checkins/' + checkin_id + '/like', querystring.stringify(this.config));
                return result;
            }
            catch (error) {
                this.error(error);
                return;
            }
        });
    }
    likeUnliked(limit = 40) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const succeeded = [];
                const recent = yield this.getRecent(limit);
                for (const checkin of recent) {
                    this.log(`Checkin ${checkin.id} liked before = ${checkin.like}`);
                    if (checkin.like == false) {
                        const liked_result = yield this.likeCheckin(checkin.id);
                        this.log(`liked ${checkin.id}`);
                        succeeded.push(liked_result);
                    }
                }
                return succeeded;
            }
            catch (error) {
                this.error(error);
            }
        });
    }
    // Utility functions
    between(min, max) {
        return Math.floor(Math.random() * (max - min) + min);
    }
    createLatLngString(lat, lng) {
        return `${lat},${lng}`;
    }
}
module.exports = SwarmappApi;
